@@ -24,7 +24,9 @@
 #endif
 #include <sys/kobj.h>
 
-
+#ifdef AFS_SUN58_ENV
+# include <sys/mount.h>
+#endif
 
 struct vfs *afs_globalVFS = 0;
 struct vcache *afs_globalVp = 0;
@@ -69,6 +71,18 @@ afs_mount(struct vfs *afsp, struct vnode *amvp, struct mounta *uap,
     return 0;
 }
 
+static void
+afs_freevfs(void)
+{
+    int i;
+    struct vcache *vc, *nvc;
+    extern struct vcache *afs_vhashT[VCSIZE];
+
+    afs_globalVFS = 0;
+
+    afs_shutdown();
+}
+
 #if defined(AFS_SUN58_ENV)
 int
 afs_unmount(struct vfs *afsp, int flag, afs_ucred_t *credp)
@@ -77,6 +91,8 @@ int
 afs_unmount(struct vfs *afsp, afs_ucred_t *credp)
 #endif
 {
+    struct vcache *rootvp = NULL;
+
     AFS_GLOCK();
     AFS_STATCNT(afs_unmount);
 
@@ -88,19 +104,65 @@ afs_unmount(struct vfs *afsp, afs_ucred_t *credp)
         AFS_GUNLOCK();
         return (EPERM);
     }
-    afs_globalVFS = 0;
-    afs_shutdown();
+
+#ifdef AFS_SUN58_ENV
+    if (flag & MS_FORCE) {
+	AFS_GUNLOCK();
+	return ENOTSUP;
+    }
+
+    /* We should have one reference from the caller, and one reference for the
+     * root vnode; any more and someone is still referencing something */
+    if (afsp->vfs_count > 2) {
+	AFS_GUNLOCK();
+	return EBUSY;
+    }
+
+    /* The root vnode should have one ref for the mount; any more, and someone
+     * else is using the root vnode */
+    if (afs_globalVp && VREFCOUNT_GT(afs_globalVp, 1)) {
+	AFS_GUNLOCK();
+	return EBUSY;
+    }
+
+    afsp->vfs_flag |= VFS_UNMOUNTED;
+#endif /* AFS_SUN58_ENV */
+
+    if (afs_globalVp) {
+	/* release the root vnode, which should be the last reference to us
+	 * besides the caller of afs_unmount */
+	rootvp = afs_globalVp;
+	afs_globalVp = NULL;
+	AFS_RELE(rootvp);
+    }
+
+#ifndef AFS_SUN58_ENV
+    /* shutdown now, since gafs_freevfs() will not be called */
+    afs_freevfs();
+#endif /* !AFS_SUN58_ENV */
 
     AFS_GUNLOCK();
     return 0;
 }
 
+#ifdef AFS_SUN58_ENV
+void
+gafs_freevfs(struct vfs *afsp)
+{
+    AFS_GLOCK();
+
+    afs_freevfs();
+
+    AFS_GUNLOCK();
+}
+#endif /* AFS_SUN58_ENV */
+
 int
 afs_root(struct vfs *afsp, struct vnode **avpp)
 {
-    register afs_int32 code = 0;
+    afs_int32 code = 0;
     struct vrequest treq;
-    register struct vcache *tvp = 0;
+    struct vcache *tvp = 0;
     struct vcache *gvp;
     struct proc *proc = ttoproc(curthread);
     struct vnode *vp = afsp->vfs_vnodecovered;
@@ -150,7 +212,7 @@ again:
 	}
     }
     if (tvp) {
-	VN_HOLD(AFSTOV(tvp));
+	AFS_FAST_HOLD(tvp);
 	mutex_enter(&AFSTOV(tvp)->v_lock);
 	AFSTOV(tvp)->v_flag |= VROOT;
 	mutex_exit(&AFSTOV(tvp)->v_lock);
@@ -188,10 +250,9 @@ afs_statvfs(struct vfs *afsp, struct statvfs *abp)
     AFS_STATCNT(afs_statfs);
 
     abp->f_frsize = 1024;
-    abp->f_favail = 9000000;
     abp->f_bsize = afsp->vfs_bsize;
     abp->f_blocks = abp->f_bfree = abp->f_bavail = abp->f_files =
-	abp->f_ffree = 9000000;
+	abp->f_favail = abp->f_ffree = AFS_VFS_FAKEFREE;
     abp->f_fsid = (AFS_VFSMAGIC << 16) || AFS_VFSFSID;
 
     AFS_GUNLOCK();
@@ -265,7 +326,7 @@ static const fs_operation_def_t afs_vfsops_template[] = {
     VFSNAME_SYNC,		{ .vfs_sync = afs_sync },
     VFSNAME_VGET,		{ .vfs_vget = afs_vget },
     VFSNAME_MOUNTROOT,  	{ .vfs_mountroot = afs_mountroot },
-    VFSNAME_FREEVFS,		{ .vfs_freevfs = fs_freevfs },
+    VFSNAME_FREEVFS,		{ .vfs_freevfs = gafs_freevfs },
     NULL,			NULL
 };
 struct vfsops *afs_vfsopsp;
@@ -279,7 +340,7 @@ const fs_operation_def_t afs_vfsops_template[] = {
     VFSNAME_SYNC,		afs_sync,
     VFSNAME_VGET,		afs_vget,
     VFSNAME_MOUNTROOT,  	afs_mountroot,
-    VFSNAME_FREEVFS,		fs_freevfs,
+    VFSNAME_FREEVFS,		gafs_freevfs,
     NULL,			NULL
 };
 struct vfsops *afs_vfsopsp;
@@ -294,7 +355,7 @@ struct vfsops Afs_vfsops = {
     afs_mountroot,
     afs_swapvp,
 #if defined(AFS_SUN58_ENV)
-    fs_freevfs,
+    gafs_freevfs,
 #endif
 };
 #endif
@@ -420,7 +481,7 @@ static vfsdef_t afs_vfsdef = {
     VFSDEF_VERSION,
     "afs",
     afsinit,
-    0,
+    VSW_STATS,
     NULL
 };
 #else
@@ -428,7 +489,7 @@ static struct vfsdef_v3 afs_vfsdef = {
     VFSDEF_VERSION,
     "afs",
     afsinit,
-    0
+    VSW_STATS
 };
 #endif
 #else
@@ -440,20 +501,18 @@ static struct vfssw afs_vfw = {
 };
 #endif
 
+#ifndef AFS_SUN511_ENV
 static struct sysent afssysent = {
     6,
     0,
     Afs_syscall
 };
-
-/* inter-module dependencies */
-char _depends_on[] = "drv/ip drv/udp strmod/rpcmod";
+#endif /* AFS_SUN511_ENV */
 
 /*
  * Info/Structs to link the afs module into the kernel
  */
 extern struct mod_ops mod_fsops;
-extern struct mod_ops mod_syscallops;
 
 static struct modlfs afsmodlfs = {
     &mod_fsops,
@@ -465,6 +524,13 @@ static struct modlfs afsmodlfs = {
 #endif
 };
 
+#ifdef AFS_SUN511_ENV
+
+extern struct modldrv afs_modldrv;
+
+#else /* AFS_SUN511_ENV */
+
+extern struct mod_ops mod_syscallops;
 static struct modlsys afsmodlsys = {
     &mod_syscallops,
     "afs syscall interface",
@@ -480,7 +546,7 @@ static struct modlsys afsmodlsys = {
   * land here from sysent or sysent32
   */
 
-#if defined(AFS_SUN57_64BIT_ENV)
+# if defined(AFS_SUN57_64BIT_ENV)
 extern struct mod_ops mod_syscallops32;
 
 static struct modlsys afsmodlsys32 = {
@@ -488,19 +554,37 @@ static struct modlsys afsmodlsys32 = {
     "afs syscall interface(32 bit)",
     &afssysent
 };
-#endif
+# endif
+#endif /* !AFS_SUN511_ENV */
 
 
 static struct modlinkage afs_modlinkage = {
     MODREV_1,
-    (void *)&afsmodlsys,
-#ifdef AFS_SUN57_64BIT_ENV
-    (void *)&afsmodlsys32,
-#endif
     (void *)&afsmodlfs,
+#ifdef AFS_SUN511_ENV
+    (void *)&afs_modldrv,
+#else
+    (void *)&afsmodlsys,
+# ifdef AFS_SUN57_64BIT_ENV
+    (void *)&afsmodlsys32,
+# endif
+#endif /* !AFS_SUN511_ENV */
     NULL
 };
 
+static void
+reset_sysent(void)
+{
+    if (afs_sinited) {
+	sysent[SYS_setgroups].sy_callc = afs_orig_setgroups;
+	sysent[SYS_ioctl].sy_call = afs_orig_ioctl;
+#if defined(AFS_SUN57_64BIT_ENV)
+	sysent32[SYS_setgroups].sy_callc = afs_orig_setgroups32;
+	sysent32[SYS_ioctl].sy_call = afs_orig_ioctl32;
+#endif
+    }
+}
+
 /** This is the function that modload calls when loading the afs kernel
   * extensions. The solaris modload program searches for the _init
   * function in a module and calls it when modloading
@@ -531,7 +615,8 @@ _init()
 #if     defined(AFS_SUN55_ENV)
     if ((!(mp = mod_find_by_filename("misc", "nfssrv"))
 	 && !(mp = mod_find_by_filename(NULL, NFSSRV))
-	 && !(mp = mod_find_by_filename(NULL, NFSSRV_V9))) || (mp
+	 && !(mp = mod_find_by_filename(NULL, NFSSRV_V9))
+	 && !(mp = mod_find_by_filename(NULL, NFSSRV_AMD64))) || (mp
 							       && !mp->
 							       mod_installed))
     {
@@ -554,7 +639,11 @@ _init()
 #endif /* AFS_SUN52_ENV */
 #endif /* AFS_SUN55_ENV */
 #endif /* !AFS_NONFSTRANS */
-#if !defined(AFS_SUN58_ENV)
+
+#ifndef AFS_SUN511_ENV
+    /* syscall initialization stff */
+
+# if !defined(AFS_SUN58_ENV)
     /* 
      * Re-read the /etc/name_to_sysnum file to make sure afs isn't added after
      * reboot.  Ideally we would like to call modctl_read_sysbinding_file() but
@@ -564,14 +653,14 @@ _init()
      * proper slot entry and we also actually have to properly initialize the
      * global sysent[AFS_SYSCALL] entry!
      */
-#ifdef	AFS_SUN53_ENV
-#ifndef	SYSBINDFILE
-#define	SYSBINDFILE	"/etc/name_to_sysnum"
-#endif /* SYSBINDFILE */
+#  ifdef	AFS_SUN53_ENV
+#   ifndef	SYSBINDFILE
+#    define	SYSBINDFILE	"/etc/name_to_sysnum"
+#   endif /* SYSBINDFILE */
     read_binding_file(SYSBINDFILE, sb_hashtab);
-#else /* !AFS_SUN53_ENV */
+#  else /* !AFS_SUN53_ENV */
     read_binding_file(sysbind, sb_hashtab);
-#endif /* AFS_SUN53_ENV */
+#  endif /* AFS_SUN53_ENV */
     make_syscallname("afs", AFS_SYSCALL);
 
     if (sysent[AFS_SYSCALL].sy_call == nosys) {
@@ -579,18 +668,24 @@ _init()
 	    sysent[AFS_SYSCALL].sy_lock =
 		(krwlock_t *) kobj_zalloc(sizeof(krwlock_t), KM_SLEEP);
 	    rw_init(sysent[AFS_SYSCALL].sy_lock, "afs_syscall",
-#ifdef AFS_SUN57_ENV
+#  ifdef AFS_SUN57_ENV
 		    RW_DEFAULT, NULL);
-#else /* !AFS_SUN57_ENV */
+#  else /* !AFS_SUN57_ENV */
 		    RW_DEFAULT, DEFAULT_WT);
-#endif /* AFS_SUN57_ENV */
+#  endif /* AFS_SUN57_ENV */
 	}
     }
-#endif /* !AFS_SUN58_ENV */
+# endif /* !AFS_SUN58_ENV */
+#endif /* !AFS_SUN511_ENV */
 
     osi_Init();			/* initialize global lock, etc */
 
     code = mod_install(&afs_modlinkage);
+    if (code) {
+	/* we failed to load, so make sure we don't leave behind any
+	 * references to our syscall handlers */
+	reset_sysent();
+    }
     return code;
 }
 
@@ -610,14 +705,7 @@ _fini()
     if (afs_globalVFS)
 	return EBUSY;
 
-    if (afs_sinited) {
-	sysent[SYS_setgroups].sy_callc = afs_orig_setgroups;
-	sysent[SYS_ioctl].sy_call = afs_orig_ioctl;
-#if defined(AFS_SUN57_64BIT_ENV)
-	sysent32[SYS_setgroups].sy_callc = afs_orig_setgroups32;
-	sysent32[SYS_ioctl].sy_call = afs_orig_ioctl32;
-#endif
-    }
+    reset_sysent();
     code = mod_remove(&afs_modlinkage);
     return code;
 }
