@@ -10,19 +10,24 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+#if defined(HAVE_LINUX_KTHREAD_RUN) && !defined(UKERNEL)
+#  include "h/kthread.h"
+#endif
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"
 #include "rx/rx_globals.h"
-#if !defined(UKERNEL) && !defined(AFS_LINUX20_ENV)
-#include "net/if.h"
-#ifdef AFS_SGI62_ENV
-#include "h/hashing.h"
-#endif
-#if !defined(AFS_HPUX110_ENV) && !defined(AFS_DARWIN_ENV)
-#include "netinet/in_var.h"
-#endif
+#if !defined(UKERNEL)
+# if !defined(AFS_LINUX20_ENV)
+#  include "net/if.h"
+#  ifdef AFS_SGI62_ENV
+#   include "h/hashing.h"
+#  endif
+#  if !defined(AFS_HPUX110_ENV) && !defined(AFS_DARWIN_ENV)
+#   include "netinet/in_var.h"
+#  endif
+# endif
 #endif /* !defined(UKERNEL) */
 #ifdef AFS_SUN510_ENV
 #include "h/ksynch.h"
@@ -73,6 +78,7 @@ static int afs_InitSetup_done = 0;
 afs_int32 afs_numcachefiles = -1;
 afs_int32 afs_numfilesperdir = -1;
 char afs_cachebasedir[1024];
+afs_int32 afs_rmtsys_enable = 0;
 
 afs_int32 afs_rx_deadtime = AFS_RXDEADTIME;
 afs_int32 afs_rx_harddead = AFS_HARDDEADTIME;
@@ -81,9 +87,8 @@ afs_int32 afs_rx_idledead_rep = AFS_IDLEDEADTIME_REP;
 
 static int afscall_set_rxpck_received = 0;
 
-#if defined(AFS_HPUX_ENV)
-extern int afs_vfs_mount();
-#endif /* defined(AFS_HPUX_ENV) */
+/* From afs_util.c */
+extern afs_int32 afs_md5inum;
 
 /* This is code which needs to be called once when the first daemon enters
  * the client. A non-zero return means an error and AFS should not start.
@@ -293,11 +298,13 @@ afsd_thread(void *rock)
 # ifdef SYS_SETPRIORITY_EXPORTED
     int (*sys_setpriority) (int, int, int) = sys_call_table[__NR_setpriority];
 # endif
-# if defined(AFS_LINUX26_ENV)
+# if !defined(HAVE_LINUX_KTHREAD_RUN)
+#  if defined(AFS_LINUX26_ENV)
     daemonize("afsd");
-# else
+#  else
     daemonize();
-# endif
+#  endif
+# endif /* !HAVE_LINUX_KTHREAD_RUN */
 				/* doesn't do much, since we were forked from keventd, but
 				 * does call mm_release, which wakes up our parent (since it
 				 * used CLONE_VFORK) */
@@ -430,8 +437,14 @@ afsd_launcher(void *rock)
     struct afsd_thread_info *rock = container_of(work, struct afsd_thread_info, tq);
 # endif
 
+# if defined(HAVE_LINUX_KTHREAD_RUN)
+    if (IS_ERR(kthread_run(afsd_thread, (void *)rock, "afsd"))) {
+	afs_warn("kthread_run failed; afs startup will not complete\n");
+    }
+# else /* !HAVE_LINUX_KTHREAD_RUN */
     if (!kernel_thread(afsd_thread, (void *)rock, CLONE_VFORK | SIGCHLD))
 	afs_warn("kernel_thread failed. afs startup will not complete\n");
+# endif /* !HAVE_LINUX_KTHREAD_RUN */
 }
 
 void
@@ -664,7 +677,9 @@ afs_syscall_call(long parm, long parm2, long parm3,
 		   sizeof(struct afs_uspc_param), code);
 	namebufsz = mvParam->bufSz;
 	param1 = afs_osi_Alloc(namebufsz);
+	osi_Assert(param1 != NULL);
 	param2 = afs_osi_Alloc(namebufsz);
+	osi_Assert(param2 != NULL);
 
 	while (afs_initState < AFSOP_START_BKG)
 	    afs_osi_Sleep(&afs_initState);
@@ -869,6 +884,7 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	 * home cell flag (0x1 bit) and the nosuid flag (0x2 bit) */
 	struct afsop_cell *tcell = afs_osi_Alloc(sizeof(struct afsop_cell));
 
+	osi_Assert(tcell != NULL);
 	code = afs_InitDynroot();
 	if (!code) {
 	    AFS_COPYIN(AFSKPTR(parm2), (caddr_t)tcell->hosts, sizeof(tcell->hosts),
@@ -891,14 +907,11 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	char *tbuffer1 = osi_AllocSmallSpace(AFS_SMALLOCSIZ);
 	int cflags = parm4;
 
+	osi_Assert(tcell != NULL);
+	osi_Assert(tbuffer != NULL);
+	osi_Assert(tbuffer1 != NULL);
 	code = afs_InitDynroot();
 	if (!code) {
-#if 0
-	    /* wait for basic init - XXX can't find any reason we need this? */
-	    while (afs_initState < AFSOP_START_BKG)
-		afs_osi_Sleep(&afs_initState);
-#endif
-
 	    AFS_COPYIN(AFSKPTR(parm2), (caddr_t)tcell->hosts, sizeof(tcell->hosts),
 		       code);
 	}
@@ -1028,10 +1041,6 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	code = 0;
 	AFS_COPYINSTR(AFSKPTR(parm2), tbuffer, AFS_SMALLOCSIZ, &bufferSize,
 		      code);
-	if (code) {
-	    osi_FreeSmallSpace(tbuffer);
-	    goto out;
-	}
 	if (!code) {
 	    tbuffer[AFS_SMALLOCSIZ - 1] = '\0';	/* null-terminate the name */
 	    /* We have the cache dir copied in.  Call the cache init routine */
@@ -1095,6 +1104,9 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	    afs_osi_Alloc(sizeof(afs_int32) * AFS_MAX_INTERFACE_ADDR);
 	int i;
 
+	osi_Assert(buffer != NULL);
+	osi_Assert(maskbuffer != NULL);
+	osi_Assert(mtubuffer != NULL);
 	/* This is a refresh */
 	if (count & 0x40000000) {
 	    count &= ~0x40000000;
@@ -1160,7 +1172,7 @@ afs_syscall_call(long parm, long parm2, long parm3,
 
 	if (refresh) {
 	    afs_CheckServers(1, NULL);     /* check down servers */
-	    afs_CheckServers(0, NULL);     /* check down servers */
+	    afs_CheckServers(0, NULL);     /* check up servers */
 	}
     }
 #ifdef	AFS_SGI53_ENV
@@ -1227,22 +1239,6 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	if (!code)
 	    AFS_COPYOUT((caddr_t) & mtu, AFSKPTR(parm3),
 			sizeof(afs_int32), code);
-#ifdef AFS_AIX32_ENV
-/* this is disabled for now because I can't figure out how to get access
- * to these kernel variables.  It's only for supporting user-mode rx
- * programs -- it makes a huge difference on the 220's in my testbed,
- * though I don't know why. The bosserver does this with /etc/no, so it's
- * being handled a different way for the servers right now.  */
-/*      {
-	static adjusted = 0;
-	extern u_long sb_max_dflt;
-	if (!adjusted) {
-	  adjusted = 1;
-	  if (sb_max_dflt < 131072) sb_max_dflt = 131072;
-	  if (sb_max < 131072) sb_max = 131072;
-	}
-      } */
-#endif /* AFS_AIX32_ENV */
     } else if (parm == AFSOP_GETMASK) {	/* parm2 == addr in net order */
 	afs_uint32 mask = 0;
 #if	!defined(AFS_SUN5_ENV)
@@ -1273,6 +1269,8 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	afs_int32 *kmsg = afs_osi_Alloc(kmsgLen);
 	char *cellname = afs_osi_Alloc(cellLen);
 
+	osi_Assert(kmsg != NULL);
+	osi_Assert(cellname != NULL);
 #ifndef UKERNEL
 	afs_osi_MaskUserLoop();
 #endif
@@ -1304,6 +1302,24 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	afscall_set_rxpck_received = 1;
     } else if (parm == AFSOP_SET_RXMAXMTU) {
 	rx_MyMaxSendSize = rx_maxReceiveSizeUser = rx_maxReceiveSize = parm2;
+    } else if (parm == AFSOP_SET_RXMAXFRAGS) {
+	rxi_nSendFrags = rxi_nRecvFrags = parm2;
+    } else if (parm == AFSOP_SET_RMTSYS_FLAG) {
+	afs_rmtsys_enable = parm2;
+	code = 0;
+    } else if (parm == AFSOP_SET_INUMCALC) {
+	switch (parm2) {
+	case AFS_INUMCALC_COMPAT:
+	    afs_md5inum = 0;
+	    code = 0;
+	    break;
+	case AFS_INUMCALC_MD5:
+	    afs_md5inum = 1;
+	    code = 0;
+	    break;
+	default:
+	    code = EINVAL;
+	}
     } else {
 	code = EINVAL;
     }
@@ -1343,7 +1359,7 @@ afs_CheckInit(void)
     return code;
 }
 
-int afs_shuttingdown = 0;
+enum afs_shutdown_state afs_shuttingdown = AFS_RUNNING;
 void
 afs_shutdown(void)
 {
@@ -1358,10 +1374,16 @@ afs_shutdown(void)
       return;
     }
 
-    if (afs_shuttingdown)
+    if (afs_shuttingdown != AFS_RUNNING)
 	return;
 
-    afs_shuttingdown = 1;
+    afs_shuttingdown = AFS_FLUSHING_CB;
+
+    /* Give up all of our callbacks if we can. */
+    afs_FlushVCBs(2);
+
+    afs_shuttingdown = AFS_SHUTDOWN;
+
     if (afs_cold_shutdown)
 	afs_warn("afs: COLD ");
     else
@@ -1440,6 +1462,7 @@ afs_shutdown(void)
     afs_warn("NetIfPoller... ");
     osi_StopNetIfPoller();
 #endif
+    rxi_FreeAllPackets();
 
     afs_termState = AFSOP_STOP_COMPLETE;
 
@@ -1481,7 +1504,7 @@ afs_shutdown(void)
     memset(&afs_stats_cmfullperf, 0, sizeof(struct afs_stats_CMFullPerf));
     afs_warn(" ALL allocated tables... ");
 
-    afs_shuttingdown = 0;
+    afs_shuttingdown = AFS_RUNNING;
     afs_warn("done\n");
 
     return;			/* Just kill daemons for now */
