@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -52,8 +52,9 @@
 
 #include "bc.h"			/*Backup Coordinator structs and defs */
 #include "bucoord_internal.h"
+#include "bucoord_prototypes.h"
 
-int localauth, interact;
+int localauth, interact, nobutcauth;
 char tcell[64];
 
 /*
@@ -62,7 +63,7 @@ char tcell[64];
 struct bc_config *bc_globalConfig;	/*Ptr to global BC configuration info */
 
 struct ubik_client *cstruct;	/* Ptr to Ubik client structure */
-struct ktc_token ttoken;	/* The token */
+time_t tokenExpires;		/* The token's expiration time */
 
 static const char *DefaultConfDir;	/*Default backup config directory */
 static int bcInit = 0;		/* backupInit called yet ? */
@@ -97,8 +98,8 @@ InitErrTabs(void)
     initialize_KTC_error_table();
 }
 
-/* 
- * got to account for the errors which are volume related but 
+/*
+ * got to account for the errors which are volume related but
  * not dealt with by standard errno and com_err stuff.
  */
 void
@@ -154,7 +155,7 @@ bc_HandleMisc(afs_int32 code)
 static int
 LineIsBlank(char *aline)
 {
-    register int tc;
+    int tc;
 
     while ((tc = *aline++))
 	if ((tc != ' ') && (tc != '\t') && (tc != '\n'))
@@ -212,7 +213,7 @@ bc_InitTextConfig(void)
 static int
 backupInit(void)
 {
-    register afs_int32 code;
+    afs_int32 code;
     static int initd = 0;	/* ever called? */
     PROCESS watcherPid;
     PROCESS pid;		/* LWP process ID */
@@ -253,7 +254,7 @@ backupInit(void)
     rx_SetRxDeadTime(60);
 
     /* VLDB initialization */
-    code = vldbClientInit(0, localauth, tcell, &cstruct, &ttoken);
+    code = vldbClientInit(0, localauth, tcell, &cstruct, &tokenExpires);
     if (code)
 	return (code);
 
@@ -297,13 +298,14 @@ backupInit(void)
  */
 
 static int
-MyBeforeProc(register struct cmd_syndesc *as, void *arock)
+MyBeforeProc(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
 
     /* Handling the command line opcode */
     if (!bcInit) {
 	localauth = ((as && as->parms[14].items) ? 1 : 0);
+	nobutcauth = ((as && as->parms[16].items) ? 1 : 0);
 	if (as && as->parms[15].items)
 	    strcpy(tcell, as->parms[15].items->data);
 	else
@@ -338,14 +340,14 @@ extern int dontExecute;		/* declared in commands.c */
 extern char *loadFile;		/* declared in commands.c */
 char lineBuffer[1024];		/* Line typed in by user or read from load file */
 
-/* 
+/*
  * This will dispatch a command.  It holds a recursive loop for the
  * "dump -file" option. This option reads backup commands from a file.
  *
  * Cannot put this code on other side of cmd_Dispatch call (in
  * commands.c) because when make a dispatch call when in a dispatch
  * call, environment is mucked up.
- * 
+ *
  * To avoid multiple processes stepping on each other in the dispatch code,
  * put a lock around it so only 1 process gets in at a time.
  */
@@ -356,7 +358,7 @@ struct Lock dispatchLock;	/* lock on the Dispatch call */
 
 afs_int32
 doDispatch(afs_int32 targc,
-	   char *targv[MAXV], 
+	   char *targv[MAXV],
 	   afs_int32 dispatchCount) /* to prevent infinite recursion */
 {
     char *sargv[MAXV];
@@ -444,6 +446,8 @@ add_std_args(struct cmd_syndesc *ts)
     cmd_AddParm(ts, "-localauth", CMD_FLAG, CMD_OPTIONAL,
 		"local authentication");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cell name");
+    cmd_AddParm(ts, "-nobutcauth", CMD_FLAG, CMD_OPTIONAL,
+		"no authentication to butc");
 }
 
 int
@@ -452,14 +456,14 @@ main(int argc, char **argv)
     char *targv[MAXV];		/*Ptr to parsed argv stuff */
     afs_int32 targc;		/*Num parsed arguments */
     afs_int32 code;		/*Return code */
-    register struct cmd_syndesc *ts;	/*Ptr to parsed command line */
+    struct cmd_syndesc *ts;	/*Ptr to parsed command line */
     int i;
 
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -714,8 +718,9 @@ main(int argc, char **argv)
 		"delete the dump from the backup database only");
     cmd_AddParm(ts, "-force", CMD_FLAG, CMD_OPTIONAL,
 		"always delete from backup database");
-    cmd_AddParm(ts, "-noexecute", CMD_FLAG, CMD_OPTIONAL,
-		"Just list the dumps");
+    cmd_AddParm(ts, "-noexecute", CMD_FLAG, CMD_OPTIONAL|CMD_HIDDEN, "");
+    cmd_AddParm(ts, "-dryrun", CMD_FLAG, CMD_OPTIONAL,
+		"list the dumps, don't delete anything");
 
     if (!interact)
 	add_std_args(ts);
