@@ -6,30 +6,30 @@
  * may copy or modify Sun RPC without charge, but are not authorized
  * to license or distribute it to anyone else except as part of a product or
  * program developed by the user.
- * 
+ *
  * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
  * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
  * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
- * 
+ *
  * Sun RPC is provided with no support and without any obligation on the
  * part of Sun Microsystems, Inc. to assist in its use, correction,
  * modification or enhancement.
- * 
+ *
  * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
  * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
  * OR ANY PART THEREOF.
- * 
+ *
  * In no event will Sun Microsystems, Inc. be liable for any lost revenue
  * or profits or other special, indirect and consequential damages, even if
  * Sun has been advised of the possibility of such damages.
- * 
+ *
  * Sun Microsystems, Inc.
  * 2550 Garcia Avenue
  * Mountain View, California  94043
  */
 
 /*
- * rpc_parse.c, Parser for the RPC protocol compiler 
+ * rpc_parse.c, Parser for the RPC protocol compiler
  * Copyright (C) 1987 Sun Microsystems, Inc.
  */
 #include <afsconfig.h>
@@ -182,7 +182,6 @@ get_definition(void)
 	break;
     case TOK_EOF:
 	return (NULL);
-	break;
     case TOK_PACKAGE:
 	def_package(defp);
 	break;
@@ -459,6 +458,9 @@ get_declaration(declaration * dec, defkind dkind)
 	}
 	dec->rel = REL_ARRAY;
 	if (peekscan(TOK_RANGLE, &tok)) {
+	    if ((dkind == DEF_INPARAM) || (dkind == DEF_INOUTPARAM)) {
+		error("input arrays must specify a max size");
+	    }
 	    dec->array_max = "~0";	/* unspecified size, use max */
 	} else {
 	    scan_num(&tok);
@@ -570,8 +572,11 @@ def_package(definition * defp)
     scan(TOK_IDENT, &tok);
     defp->def_name = tok.str;
     no_of_stat_funcs = 0;
-    if (PackageIndex++ >= MAX_PACKAGES)
+
+    PackageIndex++;
+    if (PackageIndex >= MAX_PACKAGES)
 	error("Exceeded upper limit of package statements\n");
+
     function_list_index = 0;
     PackagePrefix[PackageIndex] = tok.str;
     if (MasterPrefix == NULL)
@@ -901,8 +906,8 @@ analyze_ProcParams(definition * defp, token * tokp)
 	Proc_listp = &Proc_list->next;
 	decls = ALLOC(decl_list);
 	memset(decls, 0, sizeof(decl_list));
-    if (tokp->kind != TOK_RPAREN)
-        decls->decl = dec;
+	if (tokp->kind != TOK_RPAREN)
+	    decls->decl = dec;
 	*tailp = decls;
 	tailp = &decls->next;
     } while (tokp->kind != TOK_RPAREN);
@@ -922,7 +927,7 @@ generate_code(definition * defp, int proc_split_flag, int multi_flag)
 	if (Sflag || cflag)
 	    ss_Proc_CodeGeneration(defp);
     }
-    if (Sflag)
+    if (Sflag || (cflag && xflag && !proc_split_flag))
 	STOREVAL(&proc_defined[PackageIndex], defp);
 }
 
@@ -996,7 +1001,7 @@ hdle_param_tok(definition * defp, declaration * dec, token * tokp,
     Proc_list->component_kind = DEF_PARAM;
     Proc_list->code = alloc(250);
     Proc_list->scode = alloc(250);
-    get_declaration(dec, DEF_PARAM);
+    get_declaration(dec, par_kind);
     Proc_list->pl.param_name = dec->name;
     get1_param_type(defp, dec, &Proc_list->pl.param_type);
     print_param(dec);
@@ -1116,9 +1121,9 @@ cs_ProcName_setup(definition * defp, char *procheader, int split_flag)
 
     if (ansic_flag) {
 	if (split_flag) {
-	    first_arg = "register struct rx_call *z_call";
+	    first_arg = "struct rx_call *z_call";
 	} else {
-	    first_arg = "register struct rx_connection *z_conn";
+	    first_arg = "struct rx_connection *z_conn";
 	}
     } else {
 	if (split_flag) {
@@ -1149,7 +1154,7 @@ cs_ProcName_setup(definition * defp, char *procheader, int split_flag)
 				plist->pl.param_name);
 		    }
 		} else {
-		    f_print(fout, ", %s", plist->pl.param_name);    
+		    f_print(fout, ", %s", plist->pl.param_name);
 		    plist->pl.param_flag &= ~PROCESSED_PARAM;
 		}
 	    }
@@ -1167,9 +1172,9 @@ cs_ProcParams_setup(definition * defp, int split_flag)
 	return;
 
     if (!split_flag)
-	f_print(fout, "\tregister struct rx_connection *z_conn;\n");
+	f_print(fout, "\tstruct rx_connection *z_conn;\n");
     if (split_flag) {
-	f_print(fout, "\tregister struct rx_call *z_call;\n");
+	f_print(fout, "\tstruct rx_call *z_call;\n");
     }
     for (plist = defp->pc.plists; plist; plist = plist->next) {
 	if (plist->component_kind == DEF_PARAM
@@ -1425,7 +1430,8 @@ ss_ProcParams_setup(definition * defp, int *somefrees)
 		}
 		f_print(fout, "\t%s %s", plist->pl.param_type,
 			plist->pl.param_name);
-		*pntr = pres;
+		if (pntr)
+		    *pntr = pres;
 	    } else if (strchr(plist->pl.param_type, '*') == 0) {
 		f_print(fout, "\t%s %s", plist->pl.param_type,
 			plist->pl.param_name);
@@ -1515,13 +1521,21 @@ ss_ProcSpecial_setup(definition * defp, int *somefrees)
 		    *somefrees = 1;
 		    switch (defp1->pc.rel) {
 		    case REL_ARRAY:
-			f_print(fout, "\n\t%s.%s_val = 0;",
-				plist->pl.param_name, defp1->def_name);
-			f_print(fout, "\n\t%s.%s_len = 0;",
-				plist->pl.param_name, defp1->def_name);
 			plist->pl.string_name = alloc(40);
-			s_print(plist->pl.string_name, "%s_val",
-				defp1->def_name);
+			if (brief_flag) {
+			    f_print(fout, "\n\t%s.val = 0;",
+				    plist->pl.param_name);
+			    f_print(fout, "\n\t%s.len = 0;",
+				    plist->pl.param_name);
+			    s_print(plist->pl.string_name, "val");
+			} else {
+			    f_print(fout, "\n\t%s.%s_val = 0;",
+				    plist->pl.param_name, defp1->def_name);
+			    f_print(fout, "\n\t%s.%s_len = 0;",
+				    plist->pl.param_name, defp1->def_name);
+			    s_print(plist->pl.string_name, "%s_val",
+				    defp1->def_name);
+			}
 			break;
 		    case REL_POINTER:
 			f_print(fout, "\n\t%s = 0;", plist->pl.param_name);
@@ -1651,11 +1665,6 @@ ss_ProcTail_setup(definition * defp, int somefrees)
     for (plist = defp->pc.plists; plist; plist = plist->next) {
 	if (plist->component_kind == DEF_PARAM
 	    && (plist->pl.param_flag & FREETHIS_PARAM)) {
-	    char *dot = "", *extens = "";
-	    if (plist->pl.string_name) {
-		dot = ".";
-		extens = plist->pl.string_name;
-	    }
 	    f_print(fout, "\tif (!%s) goto fail1;\n", plist->scode);
 	}
     }
@@ -1767,7 +1776,7 @@ ucs_ProcName_setup(definition * defp, char *procheader, int split_flag)
 
     if (!cflag) {
      	if (ansic_flag) {
-	    f_print(fout, "int %s%s%s%s(register struct ubik_client *aclient, afs_int32 aflags",
+	    f_print(fout, "int %s%s%s%s(struct ubik_client *aclient, afs_int32 aflags",
 			  procheader, prefix, PackagePrefix[PackageIndex],
 			  defp->pc.proc_name);
 	} else {
@@ -1810,7 +1819,7 @@ ucs_ProcParams_setup(definition * defp, int split_flag)
     if (ansic_flag)
 	return;
 
-    f_print(fout, "\tregister struct ubik_client *aclient;\n\tafs_int32 aflags;\n");
+    f_print(fout, "\tstruct ubik_client *aclient;\n\tafs_int32 aflags;\n");
     for (plist = defp->pc.plists; plist; plist = plist->next) {
 	if (plist->component_kind == DEF_PARAM
 	    && !(plist->pl.param_flag & PROCESSED_PARAM)) {
@@ -1849,9 +1858,9 @@ ucs_ProcTail_setup(definition * defp, int split_flag)
     proc1_list *plist;
 
     f_print(fout, "{\tafs_int32 rcode, code, newHost, thisHost, i, _ucount;\n");
-    f_print(fout, "\tint chaseCount, pass, needsync, inlist;\n");
+    f_print(fout, "\tint chaseCount, pass, needsync;\n");
 #if 0 /* goes with block below */
-    f_print(fout, "\tint j;\n");
+    f_print(fout, "\tint j, inlist;\n");
 #endif
     f_print(fout, "\tstruct rx_connection *tc;\n");
     f_print(fout, "\tstruct rx_peer *rxp;\n");
@@ -1862,8 +1871,9 @@ ucs_ProcTail_setup(definition * defp, int split_flag)
     f_print(fout, "\t restart:\n");
     f_print(fout, "\torigLevel = aclient->initializationState;\n");
     f_print(fout, "\trcode = UNOSERVERS;\n");
-    f_print(fout, "\tchaseCount = inlist = needsync = 0;\n\n");
+    f_print(fout, "\tchaseCount = needsync = 0;\n\n");
 #if 0 /* We should do some sort of caching algorithm for this, but I need to think about it - shadow 26 jun 06 */
+    f_print(fout, "\tinlist = 0;\n");
     f_print(fout, "\tLOCK_UCLNT_CACHE;\n");
     f_print(fout, "\tfor (j = 0; ((j < SYNCCOUNT) && calls_needsync[j]); j++) {\n");
     f_print(fout, "\t\tif (calls_needsync[j] == (int *)%s%s%s) {\n", prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
@@ -1929,7 +1939,7 @@ ucs_ProcTail_setup(definition * defp, int split_flag)
     f_print(fout, "\t\tif ((pass == 0) && (aclient->states[_ucount] & CFLastFailed)) {\n");
     f_print(fout, "\t\t\tcontinue;       /* this guy's down */\n");
     f_print(fout, "\t\t}\n");
-    
+
     f_print(fout, "\t\trcode = %s%s%s(tc\n", prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
     for (plist = defp->pc.plists; plist; plist = plist->next) {
 	if (plist->component_kind == DEF_PARAM) {
@@ -1958,15 +1968,16 @@ ucs_ProcTail_setup(definition * defp, int split_flag)
     f_print(fout, "\t}                           /*p */\n\n");
     f_print(fout, "\tdone:\n");
     f_print(fout, "\tif (needsync) {\n");
-    f_print(fout, "\t\tif (!inlist) {          /* Remember proc call that needs sync site */\n");
+
 #if 0 /* We should do some sort of caching algorithm for this, but I need to think about it - shadow 26 jun 06 */
+    f_print(fout, "\t\tif (!inlist) {          /* Remember proc call that needs sync site */\n");
     f_print(fout, "\t\t\tLOCK_UCLNT_CACHE;\n");
     f_print(fout, "\t\t\tcalls_needsync[synccount % SYNCCOUNT] = (int *)%s%s%s;\n", prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
     f_print(fout, "\t\t\tsynccount++;\n");
     f_print(fout, "\t\t\tUNLOCK_UCLNT_CACHE;\n");
-#endif
     f_print(fout, "\t\t\tinlist = 1;\n");
     f_print(fout, "\t\t}\n");
+#endif
     f_print(fout, "\t\tif (!rcode) {           /* Remember the sync site - cmd successful */\n");
     f_print(fout, "\t\t\trxp = rx_PeerOf(aclient->conns[_ucount]);\n");
     f_print(fout, "\t\t\taclient->syncSite = rx_HostOf(rxp);\n");
@@ -2022,6 +2033,9 @@ er_ProcDeclExterns_setup(void)
     list *listp;
     definition *defp;
 
+    if ( !Sflag )
+	return;
+
     f_print(fout, "\n");
     for (listp = proc_defined[PackageIndex]; listp != NULL;
 	 listp = listp->next) {
@@ -2041,25 +2055,36 @@ er_ProcProcsArray_setup(void)
 
     if ((listp = proc_defined[PackageIndex])) {
 	defp = (definition *) listp->val;
-	if (defp->pc.proc_serverstub) {
-	    f_print(fout, "\nstatic afs_int32 (*StubProcsArray%d[])() = {%s",
-		    PackageIndex, defp->pc.proc_serverstub);
-	} else {
-	    f_print(fout,
-		    "\nstatic afs_int32 (*StubProcsArray%d[])(struct rx_call *z_call, XDR *z_xdrs) = {_%s%s%s",
-		    PackageIndex, prefix, defp->pc.proc_prefix,
-		    ((definition *) listp->val)->pc.proc_name);
-	    defp = (definition *) listp->val;
+	if ( cflag )  {
+	    f_print(fout, "\nstatic char *opnames%d[] = {\"%s%s\"",
+			PackageIndex, defp->pc.proc_prefix, defp->pc.proc_name);
+	}
+	else {
+	    if (defp->pc.proc_serverstub) {
+		f_print(fout, "\nstatic afs_int32 (*StubProcsArray%d[])() = {%s",
+			PackageIndex, defp->pc.proc_serverstub);
+	    } else {
+		f_print(fout,
+			"\nstatic afs_int32 (*StubProcsArray%d[])(struct rx_call *z_call, XDR *z_xdrs) = {_%s%s%s",
+			PackageIndex, prefix, defp->pc.proc_prefix,
+			((definition *) listp->val)->pc.proc_name);
+		defp = (definition *) listp->val;
+	    }
 	}
 	listp = listp->next;
     }
     for (; listp != NULL; listp = listp->next) {
 	defp = (definition *) listp->val;
-	if (defp->pc.proc_serverstub) {
-	    f_print(fout, ",%s", defp->pc.proc_serverstub);
-	} else {
-	    f_print(fout, ", _%s%s%s", prefix, defp->pc.proc_prefix,
-		    defp->pc.proc_name);
+	if ( cflag ) {
+	    f_print(fout, ", \"%s%s\"",defp->pc.proc_prefix,defp->pc.proc_name);
+	}
+	else {
+	    if (defp->pc.proc_serverstub) {
+		f_print(fout, ",%s", defp->pc.proc_serverstub);
+	    } else {
+		f_print(fout, ", _%s%s%s", prefix, defp->pc.proc_prefix,
+			defp->pc.proc_name);
+	    }
 	}
     }
     f_print(fout, "};\n\n");
@@ -2069,7 +2094,16 @@ er_ProcProcsArray_setup(void)
 static void
 er_ProcMainBody_setup(void)
 {
-    f_print(fout, "int %s%sExecuteRequest(register struct rx_call *z_call)\n",
+    if ( cflag ) {
+	f_print(fout, "char *%sTranslateOpCode(int op)\n{\n",
+		PackagePrefix[PackageIndex]);
+	f_print(fout, "\tif (op < %sLOWEST_OPCODE || op > %sHIGHEST_OPCODE)\n\t\treturn NULL;\n",
+		PackagePrefix[PackageIndex], PackagePrefix[PackageIndex]);
+	f_print(fout, "\treturn opnames%d[op - %sLOWEST_OPCODE];\n}\n",
+		PackageIndex, PackagePrefix[PackageIndex]);
+	return;
+    }
+    f_print(fout, "int %s%sExecuteRequest(struct rx_call *z_call)\n",
 	    prefix, PackagePrefix[PackageIndex]);
     f_print(fout, "{\n\tint op;\n");
     f_print(fout, "\tXDR z_xdrs;\n");
@@ -2090,17 +2124,23 @@ er_ProcMainBody_setup(void)
 static void
 er_HeadofOldStyleProc_setup(void)
 {
-    f_print(fout,
-	    "\nint %s%sExecuteRequest (register struct rx_call *z_call)\n",
-	    prefix,
+    if ( cflag ) {
+	f_print(fout, "char *%sTranslateOpCode(int op)\n{\n",
 	    (combinepackages ? MasterPrefix : PackagePrefix[PackageIndex]));
-    f_print(fout, "{\n");
-    f_print(fout, "\tint op;\n");
-    f_print(fout, "\tXDR z_xdrs;\n");
-    f_print(fout, "\t" "afs_int32 z_result;\n\n");
-    f_print(fout, "\txdrrx_create(&z_xdrs, z_call, XDR_DECODE);\n");
-    f_print(fout, "\tz_result = RXGEN_DECODE;\n");
-    f_print(fout, "\tif (!xdr_int(&z_xdrs, &op)) goto fail;\n");
+    }
+    else {
+	f_print(fout,
+		"\nint %s%sExecuteRequest (struct rx_call *z_call)\n",
+		prefix,
+		(combinepackages ? MasterPrefix : PackagePrefix[PackageIndex]));
+	f_print(fout, "{\n");
+	f_print(fout, "\tint op;\n");
+	f_print(fout, "\tXDR z_xdrs;\n");
+	f_print(fout, "\t" "afs_int32 z_result;\n\n");
+	f_print(fout, "\txdrrx_create(&z_xdrs, z_call, XDR_DECODE);\n");
+	f_print(fout, "\tz_result = RXGEN_DECODE;\n");
+	f_print(fout, "\tif (!xdr_int(&z_xdrs, &op)) goto fail;\n");
+    }
     f_print(fout, "\tswitch (op) {\n");
 }
 
@@ -2128,6 +2168,12 @@ er_BodyofOldStyleProc_setup(void)
 static void
 proc_er_case(definition * defp)
 {
+    if ( cflag ) {
+	f_print(fout, "\t\tcase %d:", defp->pc.proc_opcodenum);
+	f_print(fout, "\treturn \"%s%s\";\n",
+		defp->pc.proc_prefix, defp->pc.proc_name);
+	return;
+    }
     if (opcodesnotallowed[PackageIndex]) {
 	f_print(fout, "\t\tcase %d:\n", defp->pc.proc_opcodenum);
     } else {
@@ -2148,6 +2194,10 @@ static void
 er_TailofOldStyleProc_setup(void)
 {
     f_print(fout, "\t\tdefault:\n");
+    if ( cflag ) {
+	f_print(fout, "\t\t\treturn NULL;\n\t}\n}\n");
+	return;
+    }
     f_print(fout, "\t\t\tz_result = RXGEN_OPCODE;\n");
     f_print(fout, "\t\t\tbreak;\n\t}\n");
     f_print(fout, "fail:\n");
